@@ -187,3 +187,6 @@ DEFAULT_BUCKET_MARKER_LOCAL = "__local__"
 
 # user that starts the elasticsearch process if the current user is root
 OS_USER_ELASTICSEARCH = "localstack"
+
+# output string that indicates that the stack is ready
+READY_MARKER_OUTPUT = "Ready."
