@@ -5,6 +5,7 @@ import os
 import re
 import socket
 import ssl
+import threading
 from asyncio.selector_events import BaseSelectorEventLoop
 from typing import Dict
 
@@ -32,6 +33,7 @@ from localstack.utils.aws import aws_stack
 from localstack.utils.aws.aws_responses import LambdaResponse
 from localstack.utils.common import Mock, generate_ssl_cert, json_safe, path_from_url, start_thread
 from localstack.utils.server import http2_server
+from localstack.utils.serving import Server
 
 # set up logger
 LOG = logging.getLogger(__name__)
@@ -99,7 +101,6 @@ if EXTRA_CORS_ALLOWED_ORIGINS:
 
 
 class ProxyListener(object):
-
     # List of `ProxyListener` instances that are enabled by default for all requests
     DEFAULT_LISTENERS = []
 
@@ -146,15 +147,24 @@ class ProxyListener(object):
 
 
 class RegionBackend(object):
-    """Base class for region-specific backends for the different APIs."""
+    """Base class for region-specific backends for the different APIs.
+    RegionBackend lookup methods are not thread safe."""
 
     REGIONS: Dict[str, "RegionBackend"]
 
+    name: str  # name of the region
+
     @classmethod
-    def get(cls, region=None):
-        regions = cls.regions()
+    def get(cls, region: str = None) -> "RegionBackend":
         region = region or cls.get_current_request_region()
-        regions[region] = regions.get(region) or cls()
+
+        regions = cls.regions()
+        backend = regions.get(region)
+        if not backend:
+            backend = cls()
+            backend.name = region
+            regions[region] = backend
+
         return regions[region]
 
     @classmethod
@@ -487,6 +497,130 @@ class GenericProxy(object):
         return None
 
 
+class UrlMatchingForwarder(ProxyListener):
+    """
+    ProxyListener that matches URLs to a base url pattern, and if the request url matches the pattern, forwards it to
+    a forward_url. See TestUrlMatchingForwarder for how it behaves.
+    """
+
+    def __init__(self, base_url: str, forward_url: str) -> None:
+        super().__init__()
+        self.base_url = urlparse(base_url)
+        self.forward_url = urlparse(forward_url)
+
+    def forward_request(self, method, path, data, headers):
+        host = headers.get("Host", "")
+
+        if not self.matches(host, path):
+            return True
+
+        # build forward url
+        forward_url = self.build_forward_url(host, path)
+
+        # update headers
+        headers["Host"] = forward_url.netloc
+
+        # TODO: set proxy headers like x-forwarded-for?
+
+        return self.do_forward(method, forward_url.geturl(), headers, data)
+
+    def do_forward(self, method, url, headers, data):
+        return requests.request(method, url, data=data, headers=headers, stream=True, verify=False)
+
+    def matches(self, host, path):
+        # TODO: consider matching default ports (80, 443 if scheme is https). Example: http://localhost:80 matches
+        #  http://localhost) check host rule
+        if self.base_url.netloc:
+            if host != self.base_url.netloc:
+                return False
+
+        # check path components
+        if self.base_url.path == "/":
+            if path.startswith("/"):
+                return True
+
+        path_parts = path.split("/")
+        base_path_parts = self.base_url.path.split("/")
+
+        if len(base_path_parts) > len(path_parts):
+            return False
+
+        for i, component in enumerate(base_path_parts):
+            if component != path_parts[i]:
+                return False
+
+        return True
+
+    def build_forward_url(self, host, path):
+        # build forward url
+        if self.forward_url.hostname:
+            forward_host = self.forward_url.scheme + "://" + self.forward_url.netloc
+        else:
+            forward_host = host
+        forward_path_root = self.forward_url.path
+        forward_path = path[len(self.base_url.path) :]  # strip base path
+
+        # avoid double slashes
+        if forward_path and not forward_path_root.endswith("/"):
+            if not forward_path.startswith("/"):
+                forward_path = "/" + forward_path
+
+        forward_url = forward_host + forward_path_root + forward_path
+
+        return urlparse(forward_url)
+
+
+class EndpointProxy(ProxyListener):
+    def __init__(self, base_url: str, forward_url: str) -> None:
+        super().__init__()
+        self.forwarder = UrlMatchingForwarder(
+            base_url=base_url,
+            forward_url=forward_url,
+        )
+
+    def forward_request(self, method, path, data, headers):
+        return self.forwarder.forward_request(method, path, data, headers)
+
+    def register(self):
+        ProxyListener.DEFAULT_LISTENERS.append(self)
+
+    def unregister(self):
+        try:
+            ProxyListener.DEFAULT_LISTENERS.remove(self)
+        except ValueError:
+            pass
+
+
+class FakeEndpointProxyServer(Server):
+    """
+    Makes an EndpointProxy behave like a Server. You can use this to create transparent multiplexing behavior.
+    """
+
+    endpoint: EndpointProxy
+
+    def __init__(self, endpoint: EndpointProxy) -> None:
+        self.endpoint = endpoint
+        self._shutdown_event = threading.Event()
+
+        self._url = self.endpoint.forwarder.base_url
+        super().__init__(self._url.port, self._url.hostname)
+
+    @property
+    def url(self):
+        return self._url.geturl()
+
+    def do_run(self):
+        self.endpoint.register()
+        try:
+            self._shutdown_event.wait()
+        finally:
+            self.endpoint.unregister()
+
+    def do_shutdown(self):
+        self._shutdown_event.set()
+        self.endpoint.unregister()
+
+
 async def _accept_connection2(self, protocol_factory, conn, extra, sslcontext, *args, **kwargs):
     is_ssl_socket = DuplexSocket.is_ssl_socket(conn)
     if is_ssl_socket is False:
