@@ -6,6 +6,7 @@ import glob
 import hashlib
 import inspect
 import io
+import itertools
 import json
 import logging
 import os
@@ -72,6 +73,21 @@ CACHE = {}
 # lock for creating certificate files
 SSL_CERT_LOCK = threading.RLock()
 
+# regular expression for unprintable characters
+# Based on https://docs.aws.amazon.com/AWSSimpleQueueService/latest/APIReference/API_SendMessage.html
+#     #x9 | #xA | #xD | #x20 to #xD7FF | #xE000 to #xFFFD | #x10000 to #x10FFFF
+_unprintables = (
+    range(0x00, 0x09),
+    range(0x0A, 0x0A),
+    range(0x0B, 0x0D),
+    range(0x0E, 0x20),
+    range(0xD800, 0xE000),
+    range(0xFFFE, 0x10000),
+)
+REGEX_UNPRINTABLE_CHARS = re.compile(
+    f"[{re.escape(''.join(map(chr, itertools.chain(*_unprintables))))}]"
+)
+
 # user of the currently running process
 CACHED_USER = None
 
@@ -148,6 +164,7 @@ class ShellCommandThread(FuncThread):
         self.log_listener = log_listener
         self.stop_listener = stop_listener
         self.strip_color = strip_color
+        self.started = threading.Event()
         FuncThread.__init__(self, self.run_cmd, params, quiet=quiet)
 
     def run_cmd(self, params):
@@ -190,6 +207,7 @@ class ShellCommandThread(FuncThread):
                 inherit_cwd=self.inherit_cwd,
                 inherit_env=self.inherit_env,
             )
+            self.started.set()
             if outfile:
                 if outfile == subprocess.PIPE:
                     # get stdout/stderr from child process and write to parent output
@@ -447,6 +465,16 @@ class ArbitraryAccessObj:
         return ArbitraryAccessObj()
 
 
+class HashableList(list):
+    """Hashable list class that can be used with dicts or hashsets."""
+
+    def __hash__(self):
+        result = 0
+        for i in self:
+            result += hash(i)
+        return result
+
+
 # ----------------
 # UTILITY METHODS
 # ----------------
@@ -1426,6 +1454,10 @@ def str_remove(string, index, end_index=None):
     return "%s%s" % (string[:index], string[end_index:])
 
 
+def str_startswith_ignore_case(value: str, prefix: str) -> bool:
+    return value[: len(prefix)].lower() == prefix.lower()
+
+
 def last_index_of(array, value):
     """Return the last index of `value` in the given list, or -1 if it does not exist."""
     result = -1
@@ -1897,6 +1929,86 @@ class _RequestsSafe(type):
         return _wrapper
 
 
+class FileListener:
+    """
+    Platform independent `tail -f` command that calls a callback every time a new line is received on the file. If
+    use_tail_command is set (which is the default if we're not on windows and the tail command is available),
+    then a `tail -f` subprocess will be started. Otherwise the tailer library is used that uses polling with retry.
+    """
+
+    def __init__(self, file_path: str, callback: Callable[[str], None]):
+        self.file_path = file_path
+        self.callback = callback
+
+        self.thread: Optional[FuncThread] = None
+        self.started = threading.Event()
+
+        self.use_tail_command = not is_windows() and is_command_available("tail")
+
+    def start(self):
+        self.thread = self._do_start_thread()
+        self.started.wait()
+
+        if self.thread.result_future.done():
+            # this will re-raise exceptions from the run command that occurred before started was set
+            self.thread.result_future.result()
+
+    def join(self, timeout=None):
+        if self.thread:
+            self.thread.join(timeout=timeout)
+
+    def close(self):
+        if self.thread and self.thread.running:
+            self.thread.stop()
+
+        self.started.clear()
+        self.thread = None
+
+    def _do_start_thread(self) -> FuncThread:
+        if self.use_tail_command:
+            thread = self._create_tail_command_thread()
+            thread.start()
+            thread.started.wait(5)
+            self.started.set()
+        else:
+            thread = self._create_tailer_thread()
+            thread.start()
+
+        return thread
+
+    def _create_tail_command_thread(self) -> ShellCommandThread:
+        def _log_listener(line, *args, **kwargs):
+            try:
+                self.callback(line.rstrip("\r\n"))
+            except Exception:
+                pass
+
+        if not os.path.isfile(self.file_path):
+            raise FileNotFoundError
+
+        return ShellCommandThread(
+            cmd=["tail", "-f", self.file_path], quiet=False, log_listener=_log_listener
+        )
+
+    def _create_tailer_thread(self) -> FuncThread:
+        from tailer import Tailer
+
+        tailer = Tailer(open(self.file_path), end=True)
+
+        def _run_follow(*_):
+            try:
+                self.started.set()
+                for line in tailer.follow(delay=0.25):
+                    try:
+                        self.callback(line)
+                    except Exception:
+                        pass
+            finally:
+                tailer.close()
+
+        return FuncThread(func=_run_follow, on_stop=tailer.close)
+
+
 # create class-of-a-class
 class safe_requests(six.with_metaclass(_RequestsSafe)):
     pass
@@ -1986,5 +2098,20 @@ def canonicalize_bool_to_str(val: bool) -> str:
     return "true" if str(val).lower() == "true" else "false"
 
 
+def convert_to_printable_chars(value: Union[List, Dict, str]) -> str:
+    """Removes all unprintable characters from the given string."""
+    if isinstance(value, (dict, list)):
+
+        def _convert(obj, **kwargs):
+            if isinstance(obj, str):
+                return convert_to_printable_chars(obj)
+            return obj
+
+        return recurse_object(value, _convert)
+
+    result = REGEX_UNPRINTABLE_CHARS.sub("", value)
+    return result
+
+
 # Code that requires util functions from above
 CACHE_FILE_PATTERN = CACHE_FILE_PATTERN.replace("_random_dir_", short_uid())
